clear all
clc;

% Non -Gaussian smoothing
%
%  This Program is originally written by Genshiro Kitagawa (Kitagawa 1993, also Kitagawa 1987).
%  Translated into Matlab and modified by Tatsuma Wada. 
%
%
%     INPUTS:
%        NOISEV:  TYPE OF SYSTEM NOISE DENSITY  (0,1,2,3)
%        NOISEW:  TYPE OF OBSER. NOISE DENSITY  (0,1,2,3,4)
%        TAU2:    VARIANCE OR DISPERSION OF SYSTEM NOISE
%        SIG2:    VARIANCE OR DISPERSION OF OBSERVATION NOISE
%        BV:      SHAPE PARAMETER OF SYSTEM NOISE (FOR NOISEV = 2)
%        BW:      SHAPE PARAMETER OF OBSER. NOISE (FOR NOISEW = 2)
%     PARAMETERS:
%        MJ:      ADJUSTABLE DIMENSION OF Y, PS, FS, SS  (MJ >= N)
%        K:       NUMBER OF INTERVALS + 1


global y
global xmin
global xmax
%global noisev
%global noisew
global initd
global itf
global ith
global b
global outmin
global sig2
global tau2
global bw
global bv
global n
global ns
global nfe
global npe
global ff
global outmin
global outmax



% parameter
mj=500; 
k=201;  % number of the bands (d+1)


loc=zeros(mj,1);
loc(1)=0;



% read data
data=load('c:\gauss8.0\TC0807\trend.txt');
y=diff(data);
n=length(y);




% default values
noisev=0;
noisew=1;
initd=1;
itf=1;
ith=1;
b=1;
bw=0;
outmin=-1e+30;
outmax=1e+30;
ns=1;
nfe=n;
npe=n;
xmin=min(y);
xmax=max(y);

figmin=xmin;
figmax=xmax;

% initial probability
dx=(xmax-xmin)/(k-1);
%para=[mean(y),var(y)];

para=[mean(y),var(y)];

tau2=(.0001)^2;
bv=(1.0757)^2;
alpha=.9992;
parameter=[0,tau2,alpha,0,bv];
 
sig2=.4655^2;

for I=1:k
        x(I)=xmin+dx*(I-1);
        p(I)=gauss(x(I),para); %gaussian 
       % p(I)=mixture(x(I),parameter);  % mixture 
end

% normalization of constant term regarding the filtered process
p=normlz(p,k,dx);

% Estimated Parameters 



% variance of the system noise: 
%q=transp(k,dx,tau2,bv);
q=trans2(k,dx,tau2,bv,alpha);   % Mixture of Normals

% non gaussian smoothing.
[ff,loc,ss]=ngsmth(y,p,n,k,dx,xmin,q,loc);
clear trend
for i=1:npe
    for j=1:k
        s(j)=ss(j,i);
    end 
    st=pintvl(s,k,xmin,dx);
    for j=1:7
        trend(i,j) = st(j) + dx*loc(i);
    end
end


    
 qq=1:n;
t1=trend(:,1);
t2=trend(:,2);
t3=trend(:,3);
t4=trend(:,4);
t5=trend(:,5);
t6=trend(:,6);
t7=trend(:,7);

   
set(0,'lang','english')
% set(0,'defaultAxesFontSize',10);
% set(0,'defaultAxesFontName','times');
% set(0,'defaultTextFontSize',10);
% set(0,'defaultTextFontName','times');
set(0,'DefaultFigurePaperType','a4letter') 
set(0,'DefaultFigurePaperOrientation','Landscape');
set(0,'DefaultFigurePaperPositionMode','auto');
set(0,'defaultAxesFontName','arial');
set(0,'defaultTextFontName','arial');

ye=1947.5:.25:1998.25;
%ye=1947.50:.25:2004.25;
%ye=1954.25:.25:1998.25;

maxc=1.2;
minc=0.4;

m=[maxc,minc,minc,maxc];
change=[1973,1973,1973,1973];
figure
fill (change,m,'y');hold on;
plot(ye,t1,':',ye,t2,'-.',ye,t3,'--',ye,t4,'-',ye,t5,'--',ye,t6,'-.',ye,t7,':')
plot(ye,t4,'-','LineWidth',2.5)
axis([1947.50,1998.25,minc,maxc])
%axis([1947.50,2004.25,minc,maxc])
xlabel('year')
ylabel('slope')