function ff=ngfilt(y,f,n,k,dx,xmin,q,loc)
%  ...  NON-GAUSSIAN filtering: Constructing Likelihood  ...
%
%     INPUTS:
%       Y(I):   TIME SERIES
%       P(I):   INITIAL DENSITY
%       N:      DATA LENGTH
%       K:      NUMBER OF INTERVALS IN STEP FUNCTION APPROXIMATION
%       DX:     WIDTH OF INTERVAL
%       XMIN:   MINIMUM OF THE INTERVAL
%       Q:      SYSTEM NOISE DENSITY
%     OUTPUTS:
%       FF:     LOG-LIKELIHOOD
%       LOC(I): LOCATION OF THE CENTER OF THE INTERVAL AT STEP I
%       SS:     SMOOTHED DENSITY

ns=1;
nfe=length(y);
npe=nfe;

outmin=-1e+30;
outmax=1e+30;

ff=0;

for ii=ns:npe
% convolution (system noise)
p=convol(q,f,k);
p=normlz(p,k,dx);

% Bayes formula

  if y(ii) <= outmin  |  y(ii) >= outmax  | ii > nfe
       for i=1:k
             f(i)=p(i);
       end    
   else
       f= bayes(p,k,xmin,dx,y(ii),loc(ii));   % filtering (eq.14.11)
       [f,fint]= normlz(f,k,dx);                    % normalizing             
      
% likelihood computation
    ff=ff+log(fint);
   end
 
% saving for smoothing

%   for i=1:k
%    ps(i,ii)=p(i);
%    fs(i,ii)=f(i);
%   end
 
 % shift origin
[f,loc1] = shift(f,k,ii,n,loc); %   ????

loc=loc1;
end
