function [ff, loc2, ss]=ngsmth(y,f,n,k,dx,xmin,q,loc)
%  ...  NON-GAUSSIAN SMOOTHER  ...
%
%     INPUTS:
%       Y(I):   TIME SERIES
%       P(I):   INITIAL DENSITY
%       N:      DATA LENGTH
%       K:      NUMBER OF INTERVALS IN STEP FUNCTION APPROXIMATION
%       DX:     WIDTH OF INTERVAL
%       XMIN:   MINIMUM OF THE INTERVAL
%       Q:      SYSTEM NOISE DENSITY
%     OUTPUTS:
%       FF:     LOG-LIKELIHOOD
%       LOC(I): LOCATION OF THE CENTER OF THE INTERVAL AT STEP I
%       SS:     SMOOTHED DENSITY

global ns
global nfe
global npe
global outmin
global outmax

ff=0;



for ii=ns:npe
% convolution (system noise)
p=convol(q,f,k);
p=normlz(p,k,dx);

% Bayes formula

  if y(ii) <= outmin  |  y(ii) >= outmax  | ii > nfe
       for i=1:k
             f(i)=p(i);
       end    
   else
       f= bayes(p,k,xmin,dx,y(ii),loc(ii));   % filtering (eq.14.11)
       [f,fint]= normlz(f,k,dx);                    % normalizing
             
      
% likelihood computation
    ff=ff+log(fint);
   end
 
% saving for smoothing
  for i=1:k
   ps(i,ii)=p(i);
   fs(i,ii)=f(i);
  end
 
 % shift origin
[f,loc1] = shift(f,k,ii,n,loc); %   ????

loc=loc1;
end

% smoothing
for j=nfe:npe
    for i=1:k
        ss(i,j) = fs(i,j);
    end
end

for i=1:k
    s(i)=fs(i,nfe);
end

for ii=nfe-1:-1:ns
    for i=1:k
        t(i)=0;
        p(i)=0;
        f(i)=fs(i,ii);
    end
    for I=1:k
        j = I - ( loc(ii+1)-loc(ii) );
        if j >= 1 & j <= k
            p(I) = ps(j,ii+1);
      
            t(I) = s(j);
          
        end
    end
    for i=1:k
        s(i) = t(i);
    end
    t=sconvl(q,s,p,f,k);
    t=normlz(t,k,dx);
    for i=1:k
        s(i) = t(i);
        ss(i,ii)=s(i);
    end
end

loc2=loc;


